"""
Unit tests for remove function
"""

import os
import sys
from os import path
from unittest import TestCase, main

from authk._authorized_keys import _FILE_NAME
from authk.add import add
from authk.remove import remove

_KEY_TEXT = "".join(
    [
        "ssh-rsa AAAAB3NzaC1yc2EAAAABIwAAAQEAklOUpkDHrfHY17SbrmTIpNLTGK9Tjom/",
        "BWDSUGPl+nafzlHDTYW7hdI4yZ5ew18JH4JW9jbhUFrviQzM7xlELEVf4h9lFX5QVk",
        "bPppSwg0cda3Pbv7kOdJ/MTyBlWXFCR+HAo3FXRitBqxiX1nKhXpHAZsMciLq8V6Rjs",
        "NAQwdsdMFvSlVK/7XAt3FaoJoAsncM1Q9x5+3V0Ww68/eIFmb1zuUFljQJKprrX88Xyp",
        "NDvjYNby6vw/Pb0rwert/EnmZ+AW4OZPnTPI89ZPmVMLuayrD2cE86Z/il8b+gw3r3+1",
        "nKatmIkjn2so1d01QraTlMqVSsbxNrRFi9wrf+M7Q== schacon@mylaptop.local",
    ]
)


class TestRemove(TestCase):
    """
    Unit tests for remove function
    """

    def setUp(self):
        self._key = _KEY_TEXT
        if not os.path.isdir(_FILE_NAME.replace("authorized_keys", "")):
            os.mkdir(_FILE_NAME.replace("authorized_keys", ""))
        os.close(os.open(_FILE_NAME, os.O_RDWR | os.O_CREAT))
        if os.path.isfile(_FILE_NAME):
            with open(_FILE_NAME, "w+", encoding="utf-8"):
                print(f'{_FILE_NAME.split("/.")[-1]} created')
            add(_KEY_TEXT)
        else:
            print("Something wrong happened")
            sys.exit(-1)

    def tearDown(self):
        if path.exists(_FILE_NAME):
            os.remove(_FILE_NAME)

    def test_remove(self):
        remove(self._key)


if __name__ == "__main__":
    main()



"""
Adds key to authorized_keys
"""

from typing import Optional

from sshpubkeys import SSHKey

from authk._authorized_keys import AuthorizedKeys


def add(key_txt: str) -> Optional[str]:
    """
    Add key to authorized_keys list

    Args:
        key_txt (str): key payload in text form

    Returns:
        None

    """
    return_value = None
    key = SSHKey(key_txt, strict=True)
    with AuthorizedKeys() as aks:
        existing_key = aks.get(key.comment)
        return_value = ""
        if existing_key:
            aks.setdefault(key.comment, key)
            print(f"{key.comment} exists")
            return_value = "exists"
        else:
            aks[key.comment] = key
            print(f"{key.comment} succesfully added")
            return_value = "sucessfully added"
    return f"{key.comment} {return_value}"



"""
Deals with file operations. Adds functionality for context manager.
"""

import contextlib
import os
import shutil
from pathlib import Path
from typing import Any, Dict, Final

from sshpubkeys import AuthorizedKeysFile

_SSH_DIR: Final[str] = os.path.expanduser("~/.ssh")
_FILE_NAME: Final[str] = os.path.join(_SSH_DIR, "authorized_keys")


class AuthorizedKeys:
    """
    Class for handling authorized_keys file.
    """

    def __init__(self) -> None:
        self._keys = {}

    def __enter__(self) -> Dict[str, Any]:
        with contextlib.suppress(FileNotFoundError):
            authk = AuthorizedKeysFile(open(_FILE_NAME, encoding="utf-8"))
            for key in authk.keys:
                self._keys[key.comment] = key
        return self._keys

    @staticmethod
    def _create_ssh_dir(ssh_dir: str) -> str:
        if not os.path.exists(ssh_dir):
            os.mkdir(ssh_dir)
            os.chmod(ssh_dir, 0o700)
            dir_info = Path(ssh_dir)
            shutil.chown(ssh_dir, dir_info.owner(), dir_info.group())
            print(f"Missing directory created at {ssh_dir}")
        return "Success"

    @staticmethod
    def _change_file_ownership(file_name: str) -> None:
        file_info = Path(file_name)
        shutil.chown(file_name, file_info.owner(), file_info.group())
        os.chmod(file_name, 0o600)

    def _create_or_update_authoried_keys(self, file_name: str) -> None:
        with open(file_name, "a", encoding="utf-8") as file:
            payload = "\n".join([str(key.keydata) for key in self._keys.values()])
            file.write(f"{payload}\n")
        self._change_file_ownership(file_name)

    def __exit__(self, exception_type, exception_value, traceback) -> None:
        self._create_ssh_dir(_SSH_DIR)
        self._create_or_update_authoried_keys(_FILE_NAME)
        if all([exception_type, exception_value, traceback]):
            print(exception_type, exception_value, traceback, end="\n")



"""
Removes key from authorized_keys
"""

from sshpubkeys import SSHKey

from authk._authorized_keys import AuthorizedKeys


def remove(key_txt: str) -> None:
    """
    Remove key from the authorized_keys list

    Args:
        key_txt (str): key payload in text form

    Returns:
        None

    """
    key = SSHKey(key_txt, strict=True)
    with AuthorizedKeys() as aks:
        del aks[key.comment]
    print(f"{key.comment} sucessfully removed")



"""
Unit tests for _authorized_keys
"""

import os
import sys
from os import path
from unittest import TestCase, main

from authk._authorized_keys import _FILE_NAME, AuthorizedKeys


class TestAuthorizedKey(TestCase):
    """
    Class for handling unit tests
    """

    def setUp(self):
        if not os.path.isdir(_FILE_NAME.replace("authorized_keys", "")):
            os.mkdir(_FILE_NAME.replace("authorized_keys", ""))
        os.close(os.open(_FILE_NAME, os.O_RDWR | os.O_CREAT))
        if os.path.isfile(_FILE_NAME):
            with open(_FILE_NAME, "w+", encoding="utf-8"):
                print(f'{_FILE_NAME.split("/.")[-1]} created')
        else:
            print("Something wrong happened")
            sys.exit(-1)

    def tearDown(self):
        if path.exists(_FILE_NAME):
            os.remove(_FILE_NAME)

    def test_if_file_name_exists(self):
        self.assertTrue(path.isfile(_FILE_NAME), "Not file")

    def test_if_supresses_filenotfounderror(self):
        self.tearDown()
        with AuthorizedKeys() as aks:
            print(aks, type(aks))
            print("Supressed")

    def test_if_authorizedkeys_initialized_with_empty_dict(self):
        aks = AuthorizedKeys()
        self.assertEqual(type(aks._keys), dict)


if __name__ == "__main__":
    main()



"""
Unit tests for add function
"""


import os
import sys
from os import path
from pathlib import Path
from typing import Final
from unittest import TestCase, main, mock

from authk._authorized_keys import AuthorizedKeys
from authk.add import add

_TEMP_FILE: Final[str] = os.path.expanduser(f"{Path.home()}/.temp_test/authorized_keys")

_KEY_TEXT = "".join(
    [
        "ssh-rsa AAAAB3NzaC1yc2EAAAABIwAAAQEAklOUpkDHrfHY17SbrmTIpNLTGK9Tjom/",
        "BWDSUGPl+nafzlHDTYW7hdI4yZ5ew18JH4JW9jbhUFrviQzM7xlELEVf4h9lFX5QVk",
        "bPppSwg0cda3Pbv7kOdJ/MTyBlWXFCR+HAo3FXRitBqxiX1nKhXpHAZsMciLq8V6Rjs",
        "NAQwdsdMFvSlVK/7XAt3FaoJoAsncM1Q9x5+3V0Ww68/eIFmb1zuUFljQJKprrX88Xyp",
        "NDvjYNby6vw/Pb0rwert/EnmZ+AW4OZPnTPI89ZPmVMLuayrD2cE86Z/il8b+gw3r3+1",
        "nKatmIkjn2so1d01QraTlMqVSsbxNrRFi9wrf+M7Q== schacon@mylaptop.local",
    ]
)


@mock.patch("authk._authorized_keys._FILE_NAME", _TEMP_FILE)
class TestAdd(TestCase):
    """
    Unit tests for add function
    """

    def setUp(self):
        print("setting up...")
        self._key = _KEY_TEXT
        if not os.path.isdir(_TEMP_FILE.replace("authorized_keys", "")):
            os.mkdir(_TEMP_FILE.replace("authorized_keys", ""))
        os.close(os.open(_TEMP_FILE, os.O_RDWR | os.O_CREAT))
        if os.path.isfile(_TEMP_FILE):
            with open(_TEMP_FILE, "w+", encoding="utf-8"):
                print(f'{_TEMP_FILE.split("/.")[-1]} created')
        else:
            print("Something wrong happened")
            sys.exit(-1)

    def tearDown(self):
        print("tear down...")
        if path.exists(_TEMP_FILE):
            os.remove(_TEMP_FILE)

        print()

    def test_stdout_of_add(self):
        result = add(self._key)
        assert result == "schacon@mylaptop.local sucessfully added"

    def test_result_of_double_add(self):
        add(self._key)
        result = add(self._key)
        assert result == "schacon@mylaptop.local exists"

    def test_if_aks_has_get_method(self):
        with AuthorizedKeys() as aks:
            self.assertTrue("get" in dir(aks))


if __name__ == "__main__":
    main()
